Unit DCore;

interface

Uses
  DDebug,
  DUtils,
  DParser;

Type
TEvent = class;
TEntity = class;
TClassEntity = class of TEntity;
TList = class;
TDecorator = class;
TDecoratable = class;

TEvent = class
private
  FSender: TEntity;
  FID: String;
public
  Constructor Create(Sender: TEntity); 
  Function IsType(ID: String): Boolean;
  Function IsSubType(ID: String): Boolean;
  Property Sender: TEntity read FSender;
end;

TEvCreated = class(TEvent)
private
  FEntity: TEntity;
public
  Constructor Create(Sender: TEntity; Entity: TEntity);
  Property Entity: TEntity read FEntity;
end;

TEvRender = class(TEvent)
end;

TEvUpdate = class(TEvent)
end;

TEvDestroying = class(TEvCreated)
end;

TEntity = class
private
  FOwner: TEntity;
  FGrabber: Boolean;
  FHash: Cardinal;
public
  Constructor Create(Owner: TEntity); virtual;
  Procedure DoString(S: String); virtual;
  Procedure AssignVar(Name, Value: String); virtual;
  Procedure Render; virtual;
  Procedure Update; virtual;
  Procedure Kill; virtual;
  Procedure OnEvent(Event: TEvent); virtual;
  Procedure SendEvent(Event: TEvent); virtual;
  Procedure SendEventAndFree(Event: TEvent);
  Property Grabber: Boolean read FGrabber;
  Property Owner: TEntity read FOwner; 
  Property Hash: Cardinal read FHash;
end;

TList = class(TEntity)
private
  FEntities: Array Of TEntity;
public
  Procedure Render; override;
  Procedure  Update; override;
  Procedure SendEvent(Event: TEvent); override;
  Procedure GrabberCollect;
  Procedure Add(TheObject: TEntity);
  Procedure Del(TheObject: TEntity);
  Function Find(Entity: TEntity): Integer;
  Procedure FreeAll;
  Function Last: TEntity;
  Procedure Union(A, B: TList);
  Procedure AddList(Other: TList);
  Function GetEntity(Index: Integer): TEntity;
  Procedure SetEntity(Index: Integer; Entity: TEntity);
  Function Length: Integer;
  Function High: Integer;
  Procedure SetLength(Len: Integer);
  Property Entities[Index: Integer]: TEntity read GetEntity 
                                             write SetEntity; default;
end;

TNamedList = class(TEntity)
private
  FNames: Array Of String;
  FEntities: TList;
  Procedure Add(Name: String; Entity: TEntity);
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure SendEvent(Event: TEvent); override;
  Procedure FreeAll;
  Function GetEntity(Name: String): TEntity;
  Procedure SetEntity(Name: String; Entity: TEntity);
  Property Items[Index: String]: TEntity read GetEntity 
                                         write SetEntity; default;
  Property List: TList read FEntities;
end;

TDecorator = class(TEntity)
  Function GetDecoratable: TDecoratable;
  Property Decoratable: TDecoratable read GetDecoratable;
end;

TDecoratable = class(TEntity)
private
  FDecorators: TList;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure AddDecorator(Decorator: TDecorator);
  Procedure SendEvent(Event: TEvent); override;
  Function HasDecorator(DecType: TClassEntity): Boolean; overload;
  Function HasDecorator(DecType: String): Boolean; overload;
  Property Decorators: TList read FDecorators;
end;

TFactory = class(TObject)
private
  FClasses: Array Of TClassEntity;
public
  Procedure RegistClass(TheClass: TClassEntity);
  Function CreateEntity(TypeName: String; Owner: TEntity): TEntity;
end;

{// World.pas
TPhys = class(TDecorator)
  Property Objects: 
end;

TWorld = class(TDecoratable)
  Factory: TFactory;
  Objects: TList;
  Managers: TList;
end;}

Var
  Factory: TFactory;

implementation

//
//  TEvent
//
Constructor TEvent.Create;
begin
  Inherited Create;
  FID := AnsiLowerCase(ClassName);
end;

Function TEvent.IsType(ID: String): Boolean;
begin
  Result := FID = AnsiLowerCase(ID);
end;

Function TEvent.IsSubType(ID: String): Boolean;
  Var
    S: String;
begin
  S := FID;
    If Length(ID) < Length(FID) then
      SetLength(S, Length(ID));
  Result := AnsiLowerCase(ID) = S;
end;

//
//  TEvCreated
//
Constructor TEvCreated.Create(Sender: TEntity; Entity: TEntity);
begin
  Inherited Create(Sender);
  FEntity := Entity;
end;

//
//  TEntity
//
Constructor TEntity.Create(Owner: TEntity);
begin
  Inherited Create;
  FOwner := Owner;
  FGrabber := False;
  FHash := Random(4096);
end;

Procedure TEntity.DoString(S: String);
  Var
    I: Integer;
    List: TArrayOfString;
    Name, Value: String;
begin
  ParseList(S, '/', List);
    For I := 0 to High(List) do begin
      Name := StringDeletePost(List[I], Pos('=', List[I])); 
      Value := StringDeletePre(List[I], Pos('=', List[I]));
      AssignVar(Name, Value);
    end;
end;

Procedure TEntity.AssignVar(Name, Value: String);
begin
end;

Procedure TEntity.Render;
begin
end;

Procedure TEntity.Update;
begin
end;

Procedure TEntity.Kill;
begin
  FGrabber := True;
end;

Procedure TEntity.OnEvent(Event: TEvent);
begin
    If Event.IsType('TEvRender') then
      Render
    else If Event.IsType('TEvUpdate') then
      Update;         
end;

Procedure TEntity.SendEvent(Event: TEvent); 
begin
  OnEvent(Event);
end;

Procedure TEntity.SendEventAndFree(Event: TEvent);
begin
  SendEvent(Event);
  Event.Free;
end;

//
//  TList
//
Procedure TList.Render;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Render;
end;

Procedure TList.Update;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Update;
end;

Procedure TList.SendEvent(Event: TEvent);
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].SendEvent(Event);
end;

Procedure TList.GrabberCollect;
  Var
    I: Integer;
begin
  I := 0;
    While I < Length do begin
        If FEntities[I].Grabber then begin
          SendEventAndFree(TEvDestroying.Create(Self, FEntities[I]));
          FEntities[I].Free;
          FEntities[I] := FEntities[High];
          SetLength(Length - 1);
        end else
          Inc(I);
    end;
end;

Procedure TList.Add(TheObject: TEntity);
begin
  SetLength(Length + 1);
  FEntities[High] := TheObject;
end;

Procedure TList.Del(TheObject: TEntity);
  Var
    I: Integer;
begin
  I := 0;
    While I < Length do begin
        If FEntities[I] = TheObject then begin
          FEntities[I] := Nil;
          FEntities[I] := Last;
          SetLength(Length - 1);
        end;
      Inc(I);
    end;
end;

Function TList.Find(Entity: TEntity): Integer;
  Var
    I: Integer;
begin
    For I := 0 to High do
      If FEntities[I] = Entity then begin
        Result := I;
        Exit;
      end;
  I := -1;
end;

Procedure TList.FreeAll;
  Var
    I: Integer;
begin
    For I := 0 to High do
      FEntities[I].Free;
  SetLength(0);
end;

Function TList.Last: TEntity;
begin
  Result := FEntities[High];
end;

Procedure TList.Union(A, B: TList);
  Var
    I: Integer;
begin
  SetLength(0);
    For I := 0 to A.High do
      Add(A[I]);
    For I := 0 to B.High do
      Add(B[I]);
end;

Procedure TList.AddList(Other: TList);
  Var
    I: Integer;
begin
    For I := 0 to Other.High do
      Add(Other[I]);
end;

Function TList.GetEntity(Index: Integer): TEntity;
begin
  Result := FEntities[Index];
end;

Procedure TList.SetEntity(Index: Integer; Entity: TEntity);
begin
  FEntities[Index] := Entity;
end;

Function TList.Length: Integer;
begin
  Result := System.Length(FEntities);
end;

Function TList.High: Integer;
begin
  Result := System.High(FEntities);
end;

Procedure TList.SetLength(Len: Integer);
begin
  System.SetLength(FEntities, Len);
end;

//
//  TNamedList
//
Procedure TNamedList.Add(Name: String; Entity: TEntity);
begin
  SetLength(FNames, Length(FNames) + 1);
  FNames[High(FNames)] := Name;
  FEntities.Add(Entity);
end;

Constructor TNamedList.Create;
begin
  Inherited Create(Owner);
  SetLength(FNames, 0);
  FEntities := TList.Create(Self);
end;

Destructor TNamedList.Destroy;
begin
  FEntities.Free;
  Inherited Destroy;
end;

Procedure TNamedList.SendEvent(Event: TEvent);
begin
  Inherited SendEvent(Event);
  FEntities.SendEvent(Event);
end;

Procedure TNamedList.FreeAll;
begin
  FEntities.FreeAll;
  SetLength(FNames, 0);
  FEntities.SetLength(0);
end;

Function TNamedList.GetEntity(Name: String): TEntity;
  Var
    I: Integer;
begin
    For I := 0 to High(FNames) do
      If FNames[I] = Name then begin
        Result := FEntities[I];
        Exit;
      end;
  Result := Nil;
end;

Procedure TNamedList.SetEntity(Name: String; Entity: TEntity);
  Var
    I: Integer;
begin
    For I := 0 to High(FNames) do
      If FNames[I] = Name then begin
        FEntities[I] := Entity;
        Exit;
      end;
  Add(Name, Entity);
end;

//
//  TDecorator
//
Function TDecorator.GetDecoratable: TDecoratable;
begin
  Result := TDecoratable(Owner);
end;

//
//  TDecoratable
//
Constructor TDecoratable.Create(Owner: TEntity);
begin 
  Inherited Create(Owner);
  FDecorators := TList.Create(Self);
end;

Destructor TDecoratable.Destroy; 
  Var
    I: Integer;
begin
    For I := 0 to FDecorators.High do begin
        If Owner <> Nil then
	        Owner.SendEventAndFree(TEvDestroying.Create(Self, FDecorators[I]))
	      else
          SendEventAndFree(TEvDestroying.Create(Self, FDecorators[I]));
    end;
  FDecorators.FreeAll;
  FDecorators.Free;
  Inherited Destroy;
end;

Procedure TDecoratable.AddDecorator(Decorator: TDecorator);
begin
  FDecorators.Add(Decorator);
end;

Procedure TDecoratable.SendEvent(Event: TEvent);
begin
  FDecorators.SendEvent(Event);
end;

Function TDecoratable.HasDecorator(DecType: TClassEntity): Boolean;
  Var
    I: Integer;
begin
  Result := False;
    For I := 0 to FDecorators.High do
      If FDecorators[I] is DecType then
        Result := True;
end;

Function TDecoratable.HasDecorator(DecType: String): Boolean;
  Var
    I: Integer;
begin
  Result := False;
    For I := 0 to FDecorators.High do
      If AnsiLowerCase(FDecorators[I].ClassName) = AnsiLowerCase(DecType) then
        Result := True;
end;

//
//  TFactory
//
Procedure TFactory.RegistClass(TheClass: TClassEntity);
begin
  SetLength(FClasses, Length(FClasses)+1);
  FClasses[High(FClasses)] := TheClass;
  Log.Println('TFactory.RegistClass(' + TheClass.ClassName + ')');
end;

Function TFactory.CreateEntity(TypeName: String; Owner: TEntity): TEntity;
  Var
    I: Integer;
begin
  Result := Nil;
    For I := 0 to High(FClasses) do
      If AnsiLowerCase(TypeName) = 
         AnsiLowerCase(FClasses[I].ClassName) then
        Result := FClasses[I].Create(Owner);
end;

initialization
  Factory := TFactory.Create;
finalization
  Factory.Free;
end.
