Unit GCore;

interface

Uses
  DDebug,
  DUtils,
  DCore,
  DParser,
  
  GEvents;

Type
TGame = class;

TGameDecorator = class(TDecorator)
public
  Function GetGame: TGame;
  Property Game: TGame read GetGame;
end;

TGame = class(TEntity)
private
  FStart: Integer;
  FDecoratables: TList;
  FEventsLocked: Boolean;
  FNextFileName: String;
  FMapName: String;
  FExiting: Boolean;
public
  Constructor Create(Owner: TEntity); override;
  Destructor Destroy; override;
  Procedure Render; override;
  Procedure Update; override; 
  Procedure AddDecoratable(Decoratable: TDecoratable);
  Procedure ClearAll;
  Procedure DoString(S: String); override;
  Procedure DoScript(FileName: String);
  Procedure Load(FileName: String);
  Procedure SendEvent(Event: TEvent); override;
  Function GetTime: Integer;
  Property MapName: String read FMapName;
  Property Decoratables: TList read FDecoratables;
  Property Exiting: Boolean read FExiting;
end;

implementation

//
//  TGameDecotator
//
Function TGameDecorator.GetGame: TGame;
begin
  Result := TGame(Owner.Owner);
end;

//
//  TGame
//
Constructor TGame.Create(Owner: TEntity); 
begin
  Inherited Create(Owner);
  FDecoratables := TList.Create(Self);
  FStart := -1;
  FEventsLocked := False;
  FMapName := 'Default';
  FExiting := False;
end;

Destructor TGame.Destroy;
begin
  ClearAll;
  FDecoratables.Free;
  Inherited Destroy;
end;

Procedure TGame.Render;
begin
  FDecoratables.SendEventAndFree(TEvRender.Create(Self));
end;

Procedure TGame.Update;
begin
    If FNextFileName <> '' then begin
      Load('DATA\Scripts\' + FNextFileName + '.dgm');
      FNextFileName := '';
    end;
    If FStart = -1 then
      FStart := GetTimer;
  FDecoratables.SendEventAndFree(TEvUpdate.Create(Self));
  FDecoratables.GrabberCollect;
end;

Procedure TGame.AddDecoratable(Decoratable: TDecoratable);
begin
  FDecoratables.Add(Decoratable);
end;

Procedure TGame.ClearAll;
  Var
    I: Integer;
begin
    For I := 0 to FDecoratables.High do
      If FDecoratables[I] <> Nil then
        FDecoratables[I].Kill;
  FDecoratables.GrabberCollect;
  FStart := -1;
  Assert(FDecoratables.Length = 0, 'Assert error: FDecoratables.Length = 0');
end;

Procedure TGame.DoString(S: String);
  Var
    Decoratable: TDecoratable;
    Entity: TEntity;
    L, R: String;
    I: Integer;
    DecList: TArrayOfString;
    EntityClass, Params: String;
begin
        If (Length(S) > 0) And (S[1] = '@') then begin
          ParseBinary(S, '=', L, R);
          L := StringDeletePre(L, 1);
          L := DeleteLeftSpaces(DeleteRightSpaces(L));
          R := DeleteLeftSpaces(DeleteRightSpaces(R));
          ParseList(R, ',', DecList);

          Decoratable := TDecoratable.Create(Self);
            For I := 0 to High(DecList) do begin
              ParseBinary(DecList[I], '!', EntityClass, Params);
              EntityClass := DeleteLeftSpaces(DeleteRightSpaces(EntityClass));
              Entity := Factory.CreateEntity(EntityClass, Decoratable);
                If (Entity = Nil) Or Not(Entity is TDecorator) then begin
                  Log.PrintLn('Error[TGame.Load]: decorator ' + DecList[I] +
                              ' not created');
                end else begin
                  Decoratable.AddDecorator(TDecorator(Entity));
                  SendEventAndFree(TEvCreated.Create(Self, Entity));
                  Entity.DoString(Params);
                end;
            end;
          AddDecoratable(Decoratable);
        end;

        If (Length(S) > 0) And (Pos('^', S) = 1) then begin
          FNextFileName := StringDeletePre(S, 1);
          FMapName := FNextFileName;
          //DoScript('DATA\Scripts\' + S + '.dgm');
        end;

        If Pos('Clear', S) = 1 then begin
          ClearAll;
        end;

        If Pos('Exit', S) = 1 then begin
          FExiting := True;
        end;
end;

Procedure TGame.DoScript(FileName: String);
  Var
    F: TextFile;
    S: String;
begin
  Writeln('DoScript: ', FileName);
  Assign(F, FileName);
  Reset(F);
    While Not Eof(F) do begin
      Readln(F, S);
      DoString(S);
    end;
  Close(F);
end;


Procedure TGame.Load(FileName: String);
begin
  DoScript(FileName);
  FMapName := FileName;
    While Pos('\', FMapName) <> 0 do
      FMapName := StringDeletePre(FMapName, Pos('\', FMapName));
  FMapName := StringDeletePost(FMapName, Pos('.', FMapName));
  Log.PrintLn('Loading... ' + FMapName);
end;

Procedure TGame.SendEvent(Event: TEvent);
begin
    If FEventsLocked then
      Exit;
  FDecoratables.SendEvent(Event);
end;

Function TGame.GetTime: Integer;
begin
  Result := GetTimer - FStart;
end;

end.
